from django.apps import AppConfig

from . import __version__


class KillmailsConfig(AppConfig):
    name = "killtracker"
    label = "killtracker"
    verbose_name = f"Killtracker v{__version__}"

    def ready(self) -> None:
        from .core.killmails import Killmail

        Killmail.reset_lock_key()



from django.test import TestCase

from ..exceptions import WebhookTooManyRequests


class TestExceptions(TestCase):
    def test_exception(self):
        ex = WebhookTooManyRequests(10)
        self.assertEqual(ex.retry_after, 10)

        ex = WebhookTooManyRequests()
        self.assertEqual(ex.retry_after, 600)



# Generated by Django 3.1.10 on 2021-05-25 17:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("authentication", "0017_remove_fleetup_permission"),
        ("killtracker", "0002_fix_webhook_notes_field"),
    ]

    operations = [
        migrations.AddField(
            model_name="tracker",
            name="exclude_attacker_states",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="exclude killmails with characters belonging to users with these Auth states",
                related_name="_tracker_exclude_attacker_states_+",
                to="authentication.State",
            ),
        ),
        migrations.AddField(
            model_name="tracker",
            name="require_attacker_states",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="only include killmails with characters belonging to users with these Auth states",
                related_name="_tracker_require_attacker_states_+",
                to="authentication.State",
            ),
        ),
        migrations.AddField(
            model_name="tracker",
            name="require_victim_states",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="only include killmails where the victim characters belong to users with these Auth states",
                related_name="_tracker_require_victim_states_+",
                to="authentication.State",
            ),
        ),
    ]



# Generated by Django 4.0.3 on 2022-06-03 13:30

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0007_evetype_description"),
        ("authentication", "0019_merge_20211026_0919"),
        ("eveonline", "0015_factions"),
        ("auth", "0012_alter_user_first_name_max_length"),
        ("killtracker", "0004_django4_update"),
    ]

    operations = [
        migrations.AddField(
            model_name="tracker",
            name="exclude_victim_alliances",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Exclude killmails where the victim belongs to one of these alliances. ",
                related_name="+",
                to="eveonline.eveallianceinfo",
            ),
        ),
        migrations.AddField(
            model_name="tracker",
            name="exclude_victim_corporations",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Exclude killmails where the victim belongs to one of these corporations. ",
                related_name="+",
                to="eveonline.evecorporationinfo",
            ),
        ),
        migrations.AddField(
            model_name="tracker",
            name="require_attacker_organizations_final_blow",
            field=models.BooleanField(
                blank=True,
                default=False,
                help_text="Only include killmails where at least one of the specified <b>required attacker corporations</b> or <b>required attacker alliances</b> has the final blow.",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="color",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Optional color for embed on Discord - #000000 / black means no color selected.",
                max_length=7,
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="exclude_attacker_alliances",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Exclude killmails with attackers from one of these alliances. ",
                related_name="+",
                to="eveonline.eveallianceinfo",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="exclude_attacker_corporations",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Exclude killmails with attackers from one of these corporations. ",
                related_name="+",
                to="eveonline.evecorporationinfo",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="exclude_attacker_states",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Exclude killmails with characters belonging to users with these Auth states. ",
                related_name="+",
                to="authentication.state",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="exclude_blue_attackers",
            field=models.BooleanField(
                default=False, help_text="Exclude killmails with blue attackers."
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="exclude_high_sec",
            field=models.BooleanField(
                default=False,
                help_text="Exclude killmails from high sec. Also exclude high sec systems in route finder for jumps from origin.",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="exclude_low_sec",
            field=models.BooleanField(
                default=False, help_text="Exclude killmails from low sec."
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="exclude_npc_kills",
            field=models.BooleanField(default=False, help_text="Exclude npc kills."),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="exclude_null_sec",
            field=models.BooleanField(
                default=False, help_text="Exclude killmails from null sec."
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="exclude_w_space",
            field=models.BooleanField(
                default=False, help_text="Exclude killmails from WH space."
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="identify_fleets",
            field=models.BooleanField(
                default=False,
                help_text="When true: kills are interpreted and shown as fleet kills.",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="is_enabled",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Toogle for activating or deactivating a tracker.",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="is_posting_name",
            field=models.BooleanField(
                default=True,
                help_text="Whether posted messages include the tracker's name.",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="name",
            field=models.CharField(
                help_text="Name to identify tracker. Will be shown on alerts posts.",
                max_length=100,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="origin_solar_system",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Solar system to calculate distance and jumps from. When provided distance and jumps will be shown on killmail messages.",
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="+",
                to="eveuniverse.evesolarsystem",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="ping_groups",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Option to ping specific group members. ",
                related_name="+",
                to="auth.group",
                verbose_name="group pings",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="ping_type",
            field=models.CharField(
                choices=[("PN", "(none)"), ("PH", "@here"), ("PE", "@everybody")],
                default="PN",
                help_text="Option to ping every member of the channel.",
                max_length=2,
                verbose_name="channel pings",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_attacker_alliances",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails with attackers from one of these alliances. ",
                related_name="+",
                to="eveonline.eveallianceinfo",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_attacker_corporations",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails with attackers from one of these corporations. ",
                related_name="+",
                to="eveonline.evecorporationinfo",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_attacker_states",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails with characters belonging to users with these Auth states. ",
                related_name="+",
                to="authentication.state",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_attackers_ship_groups",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails where at least one attacker is flying one of these ship groups. ",
                related_name="+",
                to="eveuniverse.evegroup",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_attackers_ship_types",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails where at least one attacker is flying one of these ship types. ",
                related_name="+",
                to="eveuniverse.evetype",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_blue_victim",
            field=models.BooleanField(
                default=False,
                help_text="Only include killmails where the victim has standing with our group.",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_constellations",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails that occurred in one of these regions. ",
                related_name="+",
                to="eveuniverse.eveconstellation",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_max_attackers",
            field=models.PositiveIntegerField(
                blank=True,
                default=None,
                help_text="Require killmails to have no more than max number of attackers.",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_max_distance",
            field=models.FloatField(
                blank=True,
                default=None,
                help_text="Require all killmails to be max x LY away from origin solar system.",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_max_jumps",
            field=models.PositiveIntegerField(
                blank=True,
                default=None,
                help_text="Require all killmails to be max x jumps away from origin solar system.",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_min_attackers",
            field=models.PositiveIntegerField(
                blank=True,
                default=None,
                help_text="Require killmails to have at least given number of attackers.",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_min_value",
            field=models.PositiveIntegerField(
                blank=True,
                default=None,
                help_text="Require killmail's value to be greater or equal to the given value in M ISK.",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_npc_kills",
            field=models.BooleanField(
                default=False, help_text="Only include killmails that are npc kills."
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_regions",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails that occurred in one of these regions. ",
                related_name="+",
                to="eveuniverse.everegion",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_solar_systems",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails that occurred in one of these regions. ",
                related_name="+",
                to="eveuniverse.evesolarsystem",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_victim_alliances",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails where the victim belongs to one of these alliances. ",
                related_name="+",
                to="eveonline.eveallianceinfo",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_victim_corporations",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails where the victim belongs to one of these corporations. ",
                related_name="+",
                to="eveonline.evecorporationinfo",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_victim_ship_groups",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails where victim is flying one of these ship groups. ",
                related_name="+",
                to="eveuniverse.evegroup",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_victim_ship_types",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails where victim is flying one of these ship types. ",
                related_name="+",
                to="eveuniverse.evetype",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="require_victim_states",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Only include killmails where the victim characters belong to users with these Auth states. ",
                related_name="+",
                to="authentication.state",
            ),
        ),
        migrations.AlterField(
            model_name="tracker",
            name="webhook",
            field=models.ForeignKey(
                help_text="Webhook URL for a channel on Discord to sent all alerts to.",
                on_delete=django.db.models.deletion.CASCADE,
                to="killtracker.webhook",
            ),
        ),
    ]



class CacheStub:
    """Stub for replacing Django cache"""

    def get(self, key, default=None, version=None):
        return None

    def get_or_set(self, key, default, timeout=None):
        return default()

    def set(self, key, value, timeout=None):
        return None



from enum import IntEnum

SESSION_KEY_TOOGLE_NPC = "killtracker_toogle_npc"
SESSION_KEY_USES_NPC = "killtracker_uses_npc"


class EveCategoryId(IntEnum):
    ENTITY = 11
    FIGHTER = 87
    SHIP = 6
    STRUCTURE = 65


class EveGroupId(IntEnum):
    MINING_DRONE = 101
    ORBITAL_INFRASTRUCTURE = 1025



from unittest.mock import patch

import celery
import dhooks_lite

from django.core.cache import cache
from django.test import TestCase
from django.test.utils import override_settings

from ..exceptions import WebhookTooManyRequests
from ..models import EveKillmail, Tracker, Webhook
from ..tasks import (
    delete_stale_killmails,
    generate_killmail_message,
    run_killtracker,
    run_tracker,
    send_messages_to_webhook,
    send_test_message_to_webhook,
    store_killmail,
)
from .testdata.helpers import LoadTestDataMixin, load_eve_killmails, load_killmail

MODULE_PATH = "killtracker.tasks"


class CeleryRequestStub(object):
    def __init__(self):
        self.retries = 0


class TestTrackerBase(LoadTestDataMixin, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.tracker_1 = Tracker.objects.create(
            name="Low Sec Only",
            exclude_high_sec=True,
            exclude_null_sec=True,
            exclude_w_space=True,
            webhook=cls.webhook_1,
        )
        cls.tracker_2 = Tracker.objects.create(
            name="High Sec Only",
            exclude_low_sec=True,
            exclude_null_sec=True,
            exclude_w_space=True,
            webhook=cls.webhook_1,
        )


@override_settings(CELERY_ALWAYS_EAGER=True)
@patch(MODULE_PATH + ".is_esi_online")
@patch(MODULE_PATH + ".delete_stale_killmails")
@patch(MODULE_PATH + ".store_killmail")
@patch(MODULE_PATH + ".Killmail.create_from_zkb_redisq")
@patch(MODULE_PATH + ".run_tracker")
class TestRunKilltracker(TestTrackerBase):
    def setUp(self) -> None:
        cache.clear()

    @staticmethod
    def my_fetch_from_zkb():
        for killmail_id in [10000001, 10000002, 10000003, None]:
            if killmail_id:
                yield load_killmail(killmail_id)
            else:
                yield None

    @patch(MODULE_PATH + ".KILLTRACKER_STORING_KILLMAILS_ENABLED", False)
    def test_should_run_normally(
        self,
        mock_run_tracker,
        mock_create_from_zkb_redisq,
        mock_store_killmail,
        mock_delete_stale_killmails,
        mock_is_esi_online,
    ):
        # given
        mock_create_from_zkb_redisq.side_effect = self.my_fetch_from_zkb()
        mock_is_esi_online.return_value = True
        self.webhook_1.error_queue.enqueue(load_killmail(10000004).asjson())
        # when
        run_killtracker.delay()
        # then
        self.assertEqual(mock_run_tracker.delay.call_count, 6)
        self.assertEqual(mock_store_killmail.si.call_count, 0)
        self.assertFalse(mock_delete_stale_killmails.delay.called)
        self.assertEqual(self.webhook_1.main_queue.size(), 1)
        self.assertEqual(self.webhook_1.error_queue.size(), 0)

    @patch(MODULE_PATH + ".KILLTRACKER_STORING_KILLMAILS_ENABLED", False)
    def test_should_stop_when_esi_is_offline(
        self,
        mock_run_tracker,
        mock_create_from_zkb_redisq,
        mock_store_killmail,
        mock_delete_stale_killmails,
        mock_is_esi_online,
    ):
        # given
        mock_create_from_zkb_redisq.side_effect = self.my_fetch_from_zkb()
        mock_is_esi_online.return_value = False
        # when
        run_killtracker.delay()
        # then
        self.assertEqual(mock_run_tracker.delay.call_count, 0)
        self.assertEqual(mock_store_killmail.si.call_count, 0)
        self.assertFalse(mock_delete_stale_killmails.delay.called)

    @patch(MODULE_PATH + ".KILLTRACKER_MAX_KILLMAILS_PER_RUN", 2)
    def test_should_stop_when_max_killmails_received(
        self,
        mock_run_tracker,
        mock_create_from_zkb_redisq,
        mock_store_killmail,
        mock_delete_stale_killmails,
        mock_is_esi_online,
    ):
        # given
        mock_create_from_zkb_redisq.side_effect = self.my_fetch_from_zkb()
        mock_is_esi_online.return_value = True
        # when
        run_killtracker.delay()
        # then
        self.assertEqual(mock_run_tracker.delay.call_count, 4)

    @patch(MODULE_PATH + ".KILLTRACKER_PURGE_KILLMAILS_AFTER_DAYS", 30)
    @patch(MODULE_PATH + ".KILLTRACKER_STORING_KILLMAILS_ENABLED", True)
    def test_can_store_killmails(
        self,
        mock_run_tracker,
        mock_create_from_zkb_redisq,
        mock_store_killmail,
        mock_delete_stale_killmails,
        mock_is_esi_online,
    ):
        # given
        mock_create_from_zkb_redisq.side_effect = self.my_fetch_from_zkb()
        mock_is_esi_online.return_value = True
        # when
        run_killtracker.delay()
        # then
        self.assertEqual(mock_run_tracker.delay.call_count, 6)
        self.assertEqual(mock_store_killmail.si.call_count, 3)
        self.assertTrue(mock_delete_stale_killmails.delay.called)


@patch(MODULE_PATH + ".send_messages_to_webhook")
@patch(MODULE_PATH + ".generate_killmail_message")
class TestRunTracker(TestTrackerBase):
    def setUp(self) -> None:
        cache.clear()

    def test_call_enqueue_for_matching_killmail(
        self, mock_enqueue_killmail_message, mock_send_messages_to_webhook
    ):
        """when killmail is matching, then generate new message from it"""
        killmail_json = load_killmail(10000001).asjson()
        run_tracker(self.tracker_1.pk, killmail_json)
        self.assertTrue(mock_enqueue_killmail_message.delay.called)
        self.assertFalse(mock_send_messages_to_webhook.delay.called)

    def test_do_nothing_when_no_matching_killmail(
        self, mock_enqueue_killmail_message, mock_send_messages_to_webhook
    ):
        """when killmail is not matching and webhook queue is empty,
        then do nothing
        """
        killmail_json = load_killmail(10000003).asjson()
        run_tracker(self.tracker_1.pk, killmail_json)
        self.assertFalse(mock_enqueue_killmail_message.delay.called)
        self.assertFalse(mock_send_messages_to_webhook.delay.called)

    def test_start_message_sending_when_queue_non_empty(
        self, mock_enqueue_killmail_message, mock_send_messages_to_webhook
    ):
        """when killmail is not matching and webhook queue is not empty,
        then start sending anyway
        """
        killmail_json = load_killmail(10000003).asjson()
        self.webhook_1.enqueue_message(content="test")
        run_tracker(self.tracker_1.pk, killmail_json)
        self.assertFalse(mock_enqueue_killmail_message.delay.called)
        self.assertTrue(mock_send_messages_to_webhook.delay.called)


@patch(MODULE_PATH + ".generate_killmail_message.retry")
@patch(MODULE_PATH + ".send_messages_to_webhook")
class TestGenerateKillmailMessage(TestTrackerBase):
    def setUp(self) -> None:
        cache.clear()
        self.retries = 0
        self.killmail_json = load_killmail(10000001).asjson()

    def my_retry(self, *args, **kwargs):
        self.retries += 1
        if self.retries > kwargs["max_retries"]:
            raise kwargs["exc"]
        generate_killmail_message(self.tracker_1.pk, self.killmail_json)

    def test_normal(self, mock_send_messages_to_webhook, mock_retry):
        """enqueue generated killmail and start sending"""
        mock_retry.side_effect = self.my_retry

        generate_killmail_message(self.tracker_1.pk, self.killmail_json)

        self.assertTrue(mock_send_messages_to_webhook.delay.called)
        self.assertEqual(self.webhook_1.main_queue.size(), 1)
        self.assertFalse(mock_retry.called)

    @patch(MODULE_PATH + ".KILLTRACKER_GENERATE_MESSAGE_MAX_RETRIES", 3)
    @patch(MODULE_PATH + ".Tracker.generate_killmail_message")
    def test_retry_until_maximum(
        self, mock_generate_killmail_message, mock_send_messages_to_webhook, mock_retry
    ):
        """when message generation fails,then retry until max retries is reached"""
        mock_retry.side_effect = self.my_retry
        mock_generate_killmail_message.side_effect = RuntimeError

        with self.assertRaises(RuntimeError):
            generate_killmail_message(self.tracker_1.pk, self.killmail_json)

        self.assertFalse(mock_send_messages_to_webhook.delay.called)
        self.assertEqual(self.webhook_1.main_queue.size(), 0)
        self.assertEqual(mock_retry.call_count, 4)


@patch(MODULE_PATH + ".send_messages_to_webhook.retry")
@patch(MODULE_PATH + ".Webhook.send_message_to_webhook")
@patch(MODULE_PATH + ".logger")
class TestSendMessagesToWebhook(TestTrackerBase):
    def setUp(self) -> None:
        cache.clear()

    def my_retry(self, *args, **kwargs):
        send_messages_to_webhook(self.webhook_1.pk)

    def test_one_message(self, mock_logger, mock_send_message_to_webhook, mock_retry):
        """when one mesage in queue, then send it and retry with delay"""
        mock_retry.side_effect = self.my_retry
        mock_send_message_to_webhook.return_value = dhooks_lite.WebhookResponse(
            {}, status_code=200
        )
        self.webhook_1.enqueue_message(content="Test message")

        send_messages_to_webhook(self.webhook_1.pk)

        self.assertEqual(mock_send_message_to_webhook.call_count, 1)
        self.assertEqual(self.webhook_1.main_queue.size(), 0)
        self.assertEqual(self.webhook_1.error_queue.size(), 0)
        self.assertEqual(mock_retry.call_count, 1)
        _, kwargs = mock_retry.call_args
        self.assertEqual(kwargs["countdown"], 2)
        self.assertFalse(mock_logger.error.called)
        self.assertFalse(mock_logger.warning.called)

    def test_three_message(self, mock_logger, mock_send_message_to_webhook, mock_retry):
        """when three mesages in queue, then sends them and returns 3"""
        mock_retry.side_effect = self.my_retry
        mock_send_message_to_webhook.return_value = dhooks_lite.WebhookResponse(
            {}, status_code=200
        )
        self.webhook_1.enqueue_message(content="Test message")
        self.webhook_1.enqueue_message(content="Test message")
        self.webhook_1.enqueue_message(content="Test message")

        send_messages_to_webhook(self.webhook_1.pk)

        self.assertEqual(mock_send_message_to_webhook.call_count, 3)
        self.assertEqual(self.webhook_1.main_queue.size(), 0)
        self.assertEqual(self.webhook_1.error_queue.size(), 0)
        self.assertTrue(mock_retry.call_count, 4)

    def test_no_messages(self, mock_logger, mock_send_message_to_webhook, mock_retry):
        """when no mesages in queue, then do nothing"""
        mock_retry.side_effect = self.my_retry
        mock_send_message_to_webhook.return_value = dhooks_lite.WebhookResponse(
            {}, status_code=200
        )

        send_messages_to_webhook(self.webhook_1.pk)

        self.assertEqual(mock_send_message_to_webhook.call_count, 0)
        self.assertEqual(self.webhook_1.main_queue.size(), 0)
        self.assertEqual(self.webhook_1.error_queue.size(), 0)
        self.assertEqual(mock_retry.call_count, 0)
        self.assertFalse(mock_logger.error.called)
        self.assertFalse(mock_logger.warning.called)

    def test_failed_message(
        self, mock_logger, mock_send_message_to_webhook, mock_retry
    ):
        """when message sending failed,
        then put message in error queue and log warning
        """
        mock_retry.side_effect = self.my_retry
        mock_send_message_to_webhook.return_value = dhooks_lite.WebhookResponse(
            {}, status_code=404
        )
        self.webhook_1.enqueue_message(content="Test message")

        send_messages_to_webhook(self.webhook_1.pk)

        self.assertEqual(mock_send_message_to_webhook.call_count, 1)
        self.assertEqual(self.webhook_1.main_queue.size(), 0)
        self.assertEqual(self.webhook_1.error_queue.size(), 1)
        self.assertTrue(mock_logger.warning.called)

    def test_abort_on_too_many_requests(
        self, mock_logger, mock_send_message_to_webhook, mock_retry
    ):
        """
        when WebhookTooManyRequests exception is raised
        then message is re-queued and retry once
        """
        mock_retry.side_effect = self.my_retry
        mock_send_message_to_webhook.side_effect = WebhookTooManyRequests(10)
        self.webhook_1.enqueue_message(content="Test message")

        send_messages_to_webhook(self.webhook_1.pk)

        self.assertEqual(mock_send_message_to_webhook.call_count, 1)
        self.assertEqual(self.webhook_1.main_queue.size(), 1)
        self.assertFalse(mock_retry.called)

    def test_log_info_if_not_enabled(
        self, mock_logger, mock_send_message_to_webhook, mock_retry
    ):
        my_webhook = Webhook.objects.create(
            name="disabled", url="dummy-url-2", is_enabled=False
        )
        send_messages_to_webhook(my_webhook.pk)

        self.assertFalse(mock_send_message_to_webhook.called)
        self.assertTrue(mock_logger.info.called)


@patch(MODULE_PATH + ".logger")
class TestStoreKillmail(TestTrackerBase):
    def test_normal(self, mock_logger):
        killmail = load_killmail(10000001)
        killmail_json = killmail.asjson()
        store_killmail(killmail_json)

        self.assertTrue(EveKillmail.objects.filter(id=10000001).exists())
        self.assertFalse(mock_logger.warning.called)

    def test_already_exists(self, mock_logger):
        load_eve_killmails([10000001])
        killmail = load_killmail(10000001)
        killmail_json = killmail.asjson()
        store_killmail(killmail_json)

        self.assertTrue(mock_logger.warning.called)


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
@patch("killtracker.models.dhooks_lite.Webhook.execute")
@patch(MODULE_PATH + ".logger")
class TestSendTestKillmailsToWebhook(TestTrackerBase):
    def setUp(self) -> None:
        self.webhook_1.main_queue.clear()

    def test_run_normal(self, mock_logger, mock_execute):
        # given
        mock_execute.return_value = dhooks_lite.WebhookResponse(dict(), status_code=200)
        # when
        with self.assertRaises(celery.exceptions.Retry):
            send_test_message_to_webhook.delay(self.webhook_1.pk)
        # then
        self.assertTrue(mock_execute.called)
        self.assertFalse(mock_logger.error.called)


@patch(MODULE_PATH + ".EveKillmail.objects.delete_stale")
class TestDeleteStaleKillmails(TestTrackerBase):
    def test_normal(self, mock_delete_stale):
        mock_delete_stale.return_value = (1, {"killtracker.EveKillmail": 1})
        delete_stale_killmails()
        self.assertTrue(mock_delete_stale.called)



# Generated by Django 4.0.6 on 2022-07-15 19:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0007_evetype_description"),
        ("killtracker", "0005_add_final_blow_clause_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="EveTypePlus",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("eveuniverse.evetype",),
        ),
    ]



import unittest
from datetime import timedelta
from unittest.mock import patch

import requests_mock
from redis.exceptions import LockError

from django.utils.dateparse import parse_datetime
from django.utils.timezone import now

from app_utils.esi_testing import BravadoOperationStub
from app_utils.testing import NoSocketsTestCase

from ..core.killmails import ZKB_API_URL, ZKB_REDISQ_URL, EntityCount, Killmail
from . import CacheStub
from .testdata.helpers import killmails_data, load_killmail

MODULE_PATH = "killtracker.core.killmails"
unittest.util._MAX_LENGTH = 1000


@requests_mock.Mocker()
@patch(MODULE_PATH + ".get_redis_client")
class TestCreateFromZkbRedisq(NoSocketsTestCase):
    def test_should_return_killmail(self, requests_mocker, mock_redis):
        # given
        requests_mocker.register_uri(
            "GET",
            ZKB_REDISQ_URL,
            status_code=200,
            json={"package": killmails_data()[10000001]},
        )
        # when
        killmail = Killmail.create_from_zkb_redisq()
        # then
        self.assertIsNotNone(killmail)
        self.assertEqual(killmail.id, 10000001)
        self.assertEqual(killmail.solar_system_id, 30004984)
        self.assertAlmostEqual(killmail.time, now(), delta=timedelta(seconds=120))
        self.assertEqual(killmail.victim.alliance_id, 3011)
        self.assertEqual(killmail.victim.character_id, 1011)
        self.assertEqual(killmail.victim.corporation_id, 2011)
        self.assertEqual(killmail.victim.damage_taken, 434)
        self.assertEqual(killmail.victim.ship_type_id, 603)
        self.assertEqual(len(killmail.attackers), 3)

        attacker_1 = killmail.attackers[0]
        self.assertEqual(attacker_1.alliance_id, 3001)
        self.assertEqual(attacker_1.character_id, 1001)
        self.assertEqual(attacker_1.corporation_id, 2001)
        self.assertEqual(attacker_1.damage_done, 434)
        self.assertEqual(attacker_1.security_status, -10)
        self.assertEqual(attacker_1.ship_type_id, 34562)
        self.assertEqual(attacker_1.weapon_type_id, 2977)

        self.assertEqual(killmail.zkb.location_id, 50012306)
        self.assertEqual(killmail.zkb.fitted_value, 10000)
        self.assertEqual(killmail.zkb.total_value, 10000)
        self.assertEqual(killmail.zkb.points, 1)
        self.assertFalse(killmail.zkb.is_npc)
        self.assertFalse(killmail.zkb.is_solo)
        self.assertFalse(killmail.zkb.is_awox)

    def test_should_return_none_when_zkb_returns_empty_package(
        self, requests_mocker, mock_redis
    ):
        # given
        requests_mocker.register_uri(
            "GET", ZKB_REDISQ_URL, status_code=200, json={"package": None}
        )
        # when
        killmail = Killmail.create_from_zkb_redisq()
        # then
        self.assertIsNone(killmail)

    def test_should_handle_zkb_data_has_no_solar_system(
        self, requests_mocker, mock_redis
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            ZKB_REDISQ_URL,
            status_code=200,
            json={"package": killmails_data()[10000402]},
        )
        # when
        killmail = Killmail.create_from_zkb_redisq()
        # then
        self.assertIsNotNone(killmail)

    def test_should_return_none_when_zkb_returns_429_error(
        self, requests_mocker, mock_redis
    ):
        # given
        requests_mocker.register_uri(
            "GET", ZKB_REDISQ_URL, status_code=429, text="429 too many requests"
        )
        # when
        killmail = Killmail.create_from_zkb_redisq()
        # then
        self.assertIsNone(killmail)

    def test_should_return_none_when_zkb_returns_general_error(
        self, requests_mocker, mock_redis
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            ZKB_REDISQ_URL,
            status_code=200,
            text="""Your IP has been banned because of excessive errors.

You can only have one request to listen.php in flight at any time, otherwise you will generate a too many requests error (429). If you have too many of these errors you will be banned automatically.""",
        )
        # when
        killmail = Killmail.create_from_zkb_redisq()
        # then
        self.assertIsNone(killmail)

    def test_should_return_none_when_zkb_does_not_return_json(
        self, requests_mocker, mock_redis
    ):
        # given
        requests_mocker.register_uri(
            "GET", ZKB_REDISQ_URL, status_code=200, text="this is not JSON"
        )
        # when
        killmail = Killmail.create_from_zkb_redisq()
        # then
        self.assertIsNone(killmail)

    def test_should_return_none_if_lock_not_acquired(self, requests_mocker, mock_redis):
        # given
        mock_redis.return_value.lock.side_effect = LockError
        # when
        killmail = Killmail.create_from_zkb_redisq()
        # then
        self.assertIsNone(killmail)


class TestKillmailSerialization(NoSocketsTestCase):
    def test_dict_serialization(self):
        killmail = load_killmail(10000001)
        dct_1 = killmail.asdict()
        killmail_2 = Killmail.from_dict(dct_1)
        self.maxDiff = None
        self.assertEqual(killmail, killmail_2)

    def test_json_serialization(self):
        killmail = load_killmail(10000001)
        json_1 = killmail.asjson()
        killmail_2 = Killmail.from_json(json_1)
        self.maxDiff = None
        self.assertEqual(killmail, killmail_2)


class TestKillmailBasics(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.killmail = load_killmail(10000001)

    def test_str(self):
        self.assertEqual(str(self.killmail), "Killmail(id=10000001)")

    def test_repr(self):
        self.assertEqual(repr(self.killmail), "Killmail(id=10000001)")

    def test_entity_ids(self):
        result = self.killmail.entity_ids()
        expected = {
            1011,
            2011,
            3011,
            603,
            30004984,
            1001,
            1002,
            1003,
            2001,
            3001,
            34562,
            2977,
            3756,
            2488,
            500001,
            500004,
        }
        self.assertSetEqual(result, expected)

    def test_should_return_attacker_alliance_ids(self):
        # when
        result = self.killmail.attackers_distinct_alliance_ids()
        # then
        self.assertSetEqual(set(result), {3001})

    def test_should_return_attacker_corporation_ids(self):
        # when
        result = self.killmail.attackers_distinct_corporation_ids()
        # then
        self.assertSetEqual(set(result), {2001})

    def test_should_return_attacker_character_ids(self):
        # when
        result = self.killmail.attackers_distinct_character_ids()
        # then
        self.assertSetEqual(set(result), {1001, 1002, 1003})

    def test_attackers_ships_types(self):
        self.assertListEqual(
            self.killmail.attackers_ship_type_ids(), [34562, 3756, 3756]
        )

    def test_ships_types(self):
        self.assertSetEqual(self.killmail.ship_type_distinct_ids(), {603, 34562, 3756})


class TestEntityCount(NoSocketsTestCase):
    def test_is_alliance(self):
        alliance = EntityCount(1, EntityCount.CATEGORY_ALLIANCE)
        corporation = EntityCount(2, EntityCount.CATEGORY_CORPORATION)

        self.assertTrue(alliance.is_alliance)
        self.assertFalse(corporation.is_alliance)

    def test_is_corporation(self):
        alliance = EntityCount(1, EntityCount.CATEGORY_ALLIANCE)
        corporation = EntityCount(2, EntityCount.CATEGORY_CORPORATION)

        self.assertFalse(alliance.is_corporation)
        self.assertTrue(corporation.is_corporation)


@patch(MODULE_PATH + ".cache", CacheStub())
@patch(MODULE_PATH + ".esi")
@requests_mock.Mocker()
class TestCreateFromZkbApi(NoSocketsTestCase):
    def test_normal(self, mock_esi, requests_mocker):
        killmail_id = 10000001
        killmail_data = killmails_data()[killmail_id]
        zkb_api_data = [
            {"killmail_id": killmail_data["killID"], "zkb": killmail_data["zkb"]}
        ]
        requests_mocker.register_uri(
            "GET",
            f"{ZKB_API_URL}killID/{killmail_id}/",
            status_code=200,
            json=zkb_api_data,
        )
        killmail_data["killmail"]["killmail_time"] = parse_datetime(
            killmail_data["killmail"]["killmail_time"]
        )
        mock_esi.client.Killmails.get_killmails_killmail_id_killmail_hash.return_value = BravadoOperationStub(
            killmail_data["killmail"]
        )

        killmail = Killmail.create_from_zkb_api(killmail_id)
        self.assertIsNotNone(killmail)
        self.assertEqual(killmail.id, killmail_id)
        self.assertAlmostEqual(killmail.time, now(), delta=timedelta(seconds=120))

        self.assertEqual(killmail.victim.alliance_id, 3011)
        self.assertEqual(killmail.victim.character_id, 1011)
        self.assertEqual(killmail.victim.corporation_id, 2011)
        self.assertEqual(killmail.victim.damage_taken, 434)
        self.assertEqual(killmail.victim.ship_type_id, 603)

        self.assertEqual(len(killmail.attackers), 3)

        attacker_1 = killmail.attackers[0]
        self.assertEqual(attacker_1.alliance_id, 3001)
        self.assertEqual(attacker_1.character_id, 1001)
        self.assertEqual(attacker_1.corporation_id, 2001)
        self.assertEqual(attacker_1.damage_done, 434)
        self.assertEqual(attacker_1.security_status, -10)
        self.assertEqual(attacker_1.ship_type_id, 34562)
        self.assertEqual(attacker_1.weapon_type_id, 2977)

        self.assertEqual(killmail.zkb.location_id, 50012306)
        self.assertEqual(killmail.zkb.fitted_value, 10000)
        self.assertEqual(killmail.zkb.total_value, 10000)
        self.assertEqual(killmail.zkb.points, 1)
        self.assertFalse(killmail.zkb.is_npc)
        self.assertFalse(killmail.zkb.is_solo)
        self.assertFalse(killmail.zkb.is_awox)



import json
from copy import deepcopy
from datetime import datetime
from hashlib import md5

from eveuniverse.models import EveEntity, EveType, EveUniverseEntityModel

from allianceauth.eveonline.models import EveAllianceInfo, EveCorporationInfo
from allianceauth.tests.auth_utils import AuthUtils

from ...core.killmails import Killmail
from ...models import EveKillmail, Webhook
from . import _currentdir
from .load_eveuniverse import load_eveuniverse  # noqa  pylint: disable=W0611


def _load_json_from_file(filename: str) -> dict:
    with open(f"{_currentdir}/{filename}.json", "r", encoding="utf-8") as f:
        data = json.load(f)

    return data


def _load_killmails_data() -> dict:
    data = dict()
    for obj in _load_json_from_file("killmails"):
        killmail_id = obj["killID"]
        obj["killmail"]["killmail_id"] = killmail_id
        obj["killmail"]["killmail_time"] = datetime.utcnow().strftime(
            "%Y-%m-%dT%H:%M:%SZ"
        )
        my_hash = md5(str(killmail_id).encode("utf8")).hexdigest()
        obj["zkb"]["hash"] = my_hash
        obj["zkb"][
            "href"
        ] = f"https://esi.evetech.net/v1/killmails/{killmail_id}/{my_hash}/"
        data[killmail_id] = obj

    return data


_killmails_data = _load_killmails_data()
_eveentities_data = _load_json_from_file("eveentities")
_evealliances_data = _load_json_from_file("evealliances")
_evecorporations_data = _load_json_from_file("evecorporations")


def killmails_data() -> dict:
    return deepcopy(_killmails_data)


def load_eveentities() -> None:
    for item in _eveentities_data:
        EveEntity.objects.update_or_create(
            id=item["id"], defaults={"name": item["name"], "category": item["category"]}
        )

    for MyModel in EveUniverseEntityModel.all_models():
        try:
            if MyModel.eve_entity_category():
                for obj in MyModel.objects.all():
                    EveEntity.objects.update_or_create(
                        id=obj.id,
                        defaults={
                            "name": obj.name,
                            "category": MyModel.eve_entity_category(),
                        },
                    )
        except AttributeError:
            pass


def load_evealliances() -> None:
    EveAllianceInfo.objects.all().delete()
    for item in _evealliances_data:
        alliance = EveAllianceInfo.objects.create(**item)
        EveEntity.objects.create(
            id=alliance.alliance_id,
            name=alliance.alliance_name,
            category=EveEntity.CATEGORY_ALLIANCE,
        )


def load_evecorporations() -> None:
    EveCorporationInfo.objects.all().delete()
    for item in _evecorporations_data:
        corporation = EveCorporationInfo.objects.create(**item)
        EveEntity.objects.create(
            id=corporation.corporation_id,
            name=corporation.corporation_name,
            category=EveEntity.CATEGORY_CORPORATION,
        )


def load_eve_killmails(killmail_ids: set = None) -> None:
    if killmail_ids:
        killmail_ids = set(killmail_ids)
    EveKillmail.objects.all().delete()
    for killmail_id, item in _killmails_data.items():
        if not killmail_ids or killmail_id in killmail_ids:
            killmail = Killmail._create_from_dict(item)
            EveKillmail.objects.create_from_killmail(killmail)


def load_killmail(killmail_id: int) -> Killmail:
    for item_id, item in _killmails_data.items():
        if killmail_id == item_id:
            return Killmail._create_from_dict(item)

    raise ValueError(f"Killmail with id {killmail_id} not found.")


class LoadTestDataMixin:
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_eveuniverse()
        load_evealliances()
        load_evecorporations()
        load_eveentities()
        cls.webhook_1 = Webhook.objects.create(
            name="Webhook 1", url="http://www.example.com/webhook_1", is_enabled=True
        )
        cls.webhook_2 = Webhook.objects.create(
            name="Webhook 2", url="http://www.example.com/webhook_2", is_enabled=False
        )
        cls.corporation_2001 = EveCorporationInfo.objects.get(corporation_id=2001)
        cls.corporation_2011 = EveCorporationInfo.objects.get(corporation_id=2011)
        cls.corporation_2021 = EveCorporationInfo.objects.get(corporation_id=2011)
        cls.alliance_3001 = EveAllianceInfo.objects.get(alliance_id=3001)
        cls.alliance_3011 = EveAllianceInfo.objects.get(alliance_id=3011)
        cls.type_merlin = EveType.objects.get(id=603)
        cls.type_svipul = EveType.objects.get(id=34562)
        cls.type_gnosis = EveType.objects.get(id=2977)
        cls.state_member = AuthUtils.get_member_state()
